# Class: docker
#
# This module manages docker
#
# === Parameters
#
# [*ensure*]
#   Passed to the docker package.
#   Defaults to present
#
# [*prerequired_packages*]
#   An array of additional packages that need to be installed to support
#   docker. Defaults change depending on the operating system.
#
# [*tcp_bind*]
#   The tcp socket to bind to in the format
#   tcp://127.0.0.1:4243
#   Defaults to undefined
#
# [*socket_bind*]
#   The unix socket to bind to. Defaults to
#   unix:///var/run/docker.sock.
#
# [*log_driver*]
#   Specify a log driver to use
#   Default is undef: let docker choose the correct one
#   Valid values: none, json-file, syslog, journald, gelf, fluentd, awslogs
#
# [*log_opts*]
#   Set customs labels and env options add additional attributes for use with logging drivers that accept them.
#   Each option takes a comma-separated list of keys. If there is collision between label and env keys, the value of the env takes precedence.
#   Defaults to undefined
#
# [*log_level*]
#   Set the logging level
#   Defaults to undef: docker defaults to info if no value specified
#   Valid values: debug, info, warn, error, fatal
#
# [*selinux_enabled*]
#   Enable selinux support. Default is false. SELinux does  not  presently
#   support  the  BTRFS storage driver.
#   Valid values: true, false
#
# [*service_state*]
#   Whether you want to docker daemon to start up
#   Defaults to running
#
# [*service_enable*]
#   Whether you want to docker daemon to start up at boot
#   Defaults to true
#
# [*dns*]
#   Custom dns server address
#   Defaults to undefined
#
# [*dns_search*]
#   Custom dns search domains
#   Defaults to undefined
#
# [*socket_group*]
#   Group ownership of the unix control socket.
#   Defaults to docker
#
# [*storage_driver*]
#   Specify a storage driver to use
#   Default is devicemapper
#   Valid value: devicemapper (the only officially supported on RedHat systems).
#
# [*dm_thinpooldev*]
#   Specifies a custom block storage device to use for the Docker thin pool.
#   Default is /dev/vg_docker/lv_dockerpool
#
# [*dm_basesize*]
#   The size to use when creating the base device, which limits the size of images and containers.
#   Default value is 100G
#
# [*dm_fs*]
#   The filesystem to use for the base image (xfs or ext4)
#   Defaults to xfs
#
# [*dm_mkfsarg*]
#   Specifies extra mkfs arguments to be used when creating the base device.
#
# [*dm_mountopt*]
#   Specifies extra mount options used when mounting the thin devices.
#
# [*dm_blocksize*]
#   A custom blocksize to use for the thin pool.
#   Default blocksize is 64K.
#   Warning: _DO NOT_ change this parameter after the lvm devices have been initialized.
#
# [*manage_package*]
#   Won't install or define the docker package, useful if you want to use your own package
#   Defaults to true
#
# [*package_name*]
#   Specify custom package name
#   Default is set on a per system basis in docker::params
#
# [*service_name*]
#   Specify custom service name
#   Default is set on a per system basis in docker::params
#
# [*use_swarm*]
#   Do you use swarm with consul/etcd backend and want to configure the Docker daemon to use it
#   Default to false
#
# [*swarm_cluster_store*]
#   Specify the swarm cluster store.
#   Defaut is consul://localhost:8500
#   Warning: Only the consul backend is actually supported
#
# [*swarm_cluster_advertise*]
#   Specify the advertise option for this machine on the network.
#   Defaut is eno16780032:2375
#
# [*swarm_cluster_store_opts*]
#   Specify the swarm cluster store opts.
#   Defaults to undefined
#
# [*daemon_labels*]
#   Set customs metadatas to the Docker daemon. Swarm can use them.
#   Syntax: [<namespace>.]<key>=<value>
#   Defaults to undefined
#
class docker(
  $ensure                      = $docker::params::ensure,
  $prerequired_packages        = $docker::params::prerequired_packages,
  $tcp_bind                    = $docker::params::tcp_bind,
  $socket_bind                 = $docker::params::socket_bind,
  $log_driver                  = $docker::params::log_driver,
  $log_opts                    = $docker::params::log_opts,
  $log_level                   = $docker::params::log_level,
  $selinux_enabled             = $docker::params::selinux_enabled,
  $service_state               = $docker::params::service_state,
  $service_enable              = $docker::params::service_enable,
  $root_dir                    = $docker::params::root_dir,
  $tmp_dir                     = $docker::params::tmp_dir,
  $dns                         = $docker::params::dns,
  $dns_search                  = $docker::params::dns_search,
  $socket_group                = $docker::params::socket_group,
  $storage_driver              = $docker::params::storage_driver,
  $dm_thinpooldev              = $docker::params::dm_thinpooldev,
  $dm_basesize                 = $docker::params::dm_basesize,
  $dm_fs                       = $docker::params::dm_fs,
  $dm_mkfsarg                  = $docker::params::dm_mkfsarg,
  $dm_mountopt                 = $docker::params::dm_mountopt,
  $dm_blocksize                = $docker::params::dm_blocksize,
  $manage_package              = $docker::params::manage_package,
  $package_name                = $docker::params::package_name,
  $service_name                = $docker::params::service_name,
  $docker_command              = $docker::params::docker_command,
  $use_swarm                   = $docker::params::use_swarm,
  $swarm_cluster_store         = $docker::params::swarm_cluster_store,
  $swarm_cluster_advertise     = $docker::params::swarm_cluster_advertise,
  $swarm_cluster_store_opts    = $docker::params::swarm_cluster_store_opts,
  $daemon_labels               = $docker::params::daemon_labels,
) inherits docker::params {

  validate_re($::osfamily, '^RedHat$', 'This module only works on Red Hat based systems.')
  validate_bool($manage_package)

  if $log_driver {
    validate_re($log_driver, '^(none|json-file|syslog|journald|gelf|fluentd|awslogsaufs)$', 'Valid values for log_driver are none, json-file, syslog, journald, gelf, fluentd, awslogs.' )
  }

  if $log_opts {
    validate_array($log_opts)
  }

  if $log_level {
    validate_re($log_level, '^(debug|info|warn|error|fatal)$', 'log_level must be one of debug, info, warn, error or fatal')
  }

  if $selinux_enabled {
    validate_re($selinux_enabled, '^(true|false)$', 'selinux_enabled must be true or false')
  }

  if $storage_driver {
    validate_re($storage_driver, '^(devicemapper)$', 'Valid value for storage_driver is only devicemapper, which is the only storage driver officially supported on RedHat systems.' )
  }

  if $dm_fs {
    validate_re($dm_fs, '^(ext4|xfs)$', 'Only ext4 and xfs are supported currently for dm_fs.')
  }

  if $use_swarm {
    validate_bool($use_swarm)
  }

  if $swarm_cluster_store {
    validate_re($swarm_cluster_store, '^consul://|etcd://', 'Only consul/etcd are currently supported as swarm backend.' )
  }

  if $daemon_labels {
    validate_array($daemon_labels)
  }

  class { 'docker::install': } ->
  class { 'docker::service': }
  contain 'docker::install'
  contain 'docker::service'
}
