# == Class: docker::service
#
# Class to manage the docker service daemon
#
# === Parameters
# [*tcp_bind*]
#   Which tcp port, if any, to bind the docker service to.
#
# [*socket_bind*]
#   Which local unix socket to bind the docker service to.
#
# [*socket_group*]
#   Which local unix socket to bind the docker service to.
#
# [*root_dir*]
#   Specify a non-standard root directory for docker.
#
# [*extra_parameters*]
#   Plain additional parameters to pass to the docker daemon
#
class docker::service (
  $docker_command              = $docker::docker_command,
  $service_name                = $docker::service_name,
  $tcp_bind                    = $docker::tcp_bind,
  $socket_bind                 = $docker::socket_bind,
  $log_driver                  = $docker::log_driver,
  $log_opts                    = $docker::log_opts,
  $log_level                   = $docker::log_level,
  $selinux_enabled             = $docker::selinux_enabled,
  $socket_group                = $docker::socket_group,
  $dns                         = $docker::dns,
  $dns_search                  = $docker::dns_search,
  $service_state               = $docker::service_state,
  $service_enable              = $docker::service_enable,
  $root_dir                    = $docker::root_dir,
  $storage_driver              = $docker::storage_driver,
  $tmp_dir                     = $docker::tmp_dir,
  $use_swarm_with_consul       = $docker::use_swarm_with_consul,
  $swarm_cluster_store         = $docker::swarm_cluster_store,
  $swarm_cluster_advertise     = $docker::swarm_cluster_advertise,
  $daemon_labels               = $docker::daemon_labels,
) {
  $dns_array = any2array($dns)
  $dns_search_array = any2array($dns_search)

  case $::osfamily {
    'RedHat': {
      $hasstatus     = undef
      $hasrestart    = undef

      # Create the docker unit override directory
      file { '/etc/systemd/system/docker.service.d':
        ensure => directory,
      }

      # Create the docker unit override file
      file { '/etc/systemd/system/docker.service.d/local.conf':
        ensure => present,
        force => true,
        content => template('docker/etc/systemd/system/docker.service.d/local.conf.erb'),
        notify => Service['docker.service'],
      }

      file { '/etc/sysconfig/docker':
        ensure  => present,
        force   => true,
        content => template("docker/etc/sysconfig/docker.erb"),
        notify  => Service['docker.service'],
      }

      file { '/etc/sysconfig/docker-network':
        ensure  => present,
        force   => true,
        content => template('docker/etc/sysconfig/docker-network.erb'),
        notify  => Service['docker.service'],
      }

      file { '/etc/sysconfig/docker-storage':
        ensure  => present,
        force   => true,
        content => template('docker/etc/sysconfig/docker-storage.erb'),
        notify  => Service['docker.service'],
      }

      file { '/etc/sysconfig/docker-swarm':
        ensure  => present,
        force   => true,
        content => template('docker/etc/sysconfig/docker-swarm.erb'),
        notify  => Service['docker.service'],
      }

    }
    default: {
      fail('Docker needs a RedHat based system.')
    }
  }

  exec { 'docker-systemd-reload':
    command     => '/usr/bin/systemctl daemon-reload',
    subscribe   => File['/etc/systemd/system/docker.service.d/local.conf'],
    refreshonly => true,
  }

  service { 'docker.service':
    ensure     => $service_state,
    name       => $service_name,
    enable     => $service_enable,
    hasstatus  => $hasstatus,
    hasrestart => $hasrestart,
  }

}
